var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from "path-browserify";
import { getAuthHeader, isWriteable } from "./util";
function isValidLFSInfoResponseData(val) {
    var _a, _b, _c, _d, _e;
    return ((_e = (_d = (_c = (_b = (_a = val.objects) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.actions) === null || _c === void 0 ? void 0 : _c.download) === null || _d === void 0 ? void 0 : _d.href) === null || _e === void 0 ? void 0 : _e.trim) !== undefined;
}
/**
 * Downloads, caches and returns a blob corresponding to given LFS pointer.
 * Uses already cached object, if size matches.
 */
export default function downloadBlobFromPointer(fs, url, auth, { info, objectPath }) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        if (fs) {
            try {
                const cached = yield (fs === null || fs === void 0 ? void 0 : fs.promises.readFile(objectPath));
                if (cached.byteLength === info.size) {
                    return cached;
                }
            }
            catch (e) {
                // Silence file not found errors (implies cache miss)
                if (e.code !== "ENOENT") {
                    throw e;
                }
            }
        }
        const authHeaders = "username" in auth ? getAuthHeader(auth) : {};
        // Request LFS transfer
        const lfsInfoRequestData = {
            operation: "download",
            transfers: ["basic"],
            objects: [info],
        };
        const lfsInfoRes = yield fetch(`${url}/info/lfs/objects/batch`, {
            method: "POST",
            headers: Object.assign(Object.assign({}, authHeaders), { Accept: "application/vnd.git-lfs+json", "Content-Type": "application/vnd.git-lfs+json" }),
            body: JSON.stringify(lfsInfoRequestData),
        });
        const lfsInfoResponseData = yield lfsInfoRes.json();
        if (isValidLFSInfoResponseData(lfsInfoResponseData)) {
            // Request the actual blob
            const downloadAction = lfsInfoResponseData.objects[0].actions.download;
            const lfsObjectDownloadURL = downloadAction.href;
            const lfsObjectDownloadHeaders = (_a = downloadAction.header) !== null && _a !== void 0 ? _a : {};
            const lfsObjectRes = yield fetch(lfsObjectDownloadURL, {
                method: "GET",
                headers: Object.assign(Object.assign({}, authHeaders), lfsObjectDownloadHeaders),
            });
            const blob = yield lfsObjectRes.blob();
            if (fs) {
                // Write LFS cache for this object, if cache path is accessible.
                if (yield isWriteable(fs, objectPath)) {
                    yield fs.promises.mkdir(path.dirname(objectPath), { recursive: true });
                    yield fs.promises.writeFile(objectPath, blob);
                }
            }
            return blob;
        }
        else {
            throw new Error("Unexpected JSON structure received for LFS download request");
        }
    });
}
//# sourceMappingURL=download.js.map