var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from "path-browserify";
import { Buffer } from "buffer";
import { SPEC_URL, toHex } from "./util";
function isValidPointerInfo(val) {
    return val.oid.trim !== undefined && typeof val.size === "number";
}
export function readPointerInfo(content) {
    const info = content
        .toString()
        .trim()
        .split("\n")
        .reduce((accum, line) => {
        const [k, v] = line.split(" ", 2);
        if (k === "oid") {
            accum[k] = v.split(":", 2)[1];
        }
        else if (k === "size") {
            accum[k] = parseInt(v, 10);
        }
        return accum;
    }, {});
    if (isValidPointerInfo(info)) {
        return info;
    }
    else {
        throw new Error("LFS pointer is incomplete or cannot be read");
    }
}
export function readPointer({ dir, gitdir = path.join(dir, ".git"), content, }) {
    const info = readPointerInfo(content);
    const objectPath = path.join(gitdir, "lfs", "objects", info.oid.substr(0, 2), info.oid.substr(2, 2), info.oid);
    return { info, objectPath };
}
/** Formats given PointerInfo for writing in Git tree. */
export function formatPointerInfo(info) {
    const lines = [
        `version ${SPEC_URL}`,
        `oid sha256:${info.oid}`,
        `size ${info.size}`,
        "",
    ];
    return Buffer.from(lines.join("\n"));
}
export function buildPointerInfo(content) {
    return __awaiter(this, void 0, void 0, function* () {
        const size = content.byteLength;
        const hash = yield crypto.subtle.digest("SHA-256", content);
        const oid = toHex(hash);
        return { oid, size };
    });
}
//# sourceMappingURL=pointers.js.map