var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { buildPointerInfo } from "./pointers";
import { getAuthHeader } from "./util";
function isValidLFSInfoResponseData(val) {
    var _a;
    const obj = (_a = val.objects) === null || _a === void 0 ? void 0 : _a[0];
    return obj && (!obj.actions || obj.actions.upload.href.trim !== undefined);
}
/**
 * Given a blob, uploads the blob to LFS server and returns a PointerInfo,
 * which the caller can then combine with object path into a Pointer
 * and commit in place of the original Git blob.
 */
export default function uploadBlobs({ headers = {}, url, auth }, contents) {
    return __awaiter(this, void 0, void 0, function* () {
        const infos = yield Promise.all(contents.map((c) => buildPointerInfo(c)));
        const authHeaders = auth ? getAuthHeader(auth) : {};
        // Request LFS transfer
        const lfsInfoRequestData = {
            operation: "upload",
            transfers: ["basic"],
            objects: infos,
        };
        const lfsInfoRes = yield fetch(`${url}/info/lfs/objects/batch`, {
            method: "POST",
            headers: Object.assign(Object.assign(Object.assign({}, headers), authHeaders), { Accept: "application/vnd.git-lfs+json" }),
            body: JSON.stringify(lfsInfoRequestData),
        });
        const lfsInfoResponseData = yield lfsInfoRes.json();
        if (!isValidLFSInfoResponseData(lfsInfoResponseData)) {
            throw new Error("Unexpected JSON structure received for LFS upload request");
        }
        yield Promise.all(lfsInfoResponseData.objects.map((object, index) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            // server already has file
            if (!object.actions)
                return;
            const { actions } = object;
            const resp = yield fetch(actions.upload.href, {
                method: "PUT",
                headers: Object.assign(Object.assign(Object.assign(Object.assign({}, headers), authHeaders), ((_a = actions.upload.header) !== null && _a !== void 0 ? _a : {})), { Accept: "application/vnd.git-lfs+json" }),
                body: contents[index],
            });
            if (!resp.ok)
                throw new Error(`Upload might have been unsuccessful, upload action yielded HTTP ${resp.status}`);
            if (actions.verify) {
                const verificationResp = yield fetch(actions.verify.href, {
                    method: "POST",
                    headers: Object.assign(Object.assign({}, ((_b = actions.verify.header) !== null && _b !== void 0 ? _b : {})), { Accept: "application/vnd.git-lfs+json" }),
                    body: JSON.stringify(infos[index]),
                });
                if (!resp.ok)
                    throw new Error(`Upload might have been unsuccessful, verification action yielded HTTP ${verificationResp.status}`);
            }
        })));
        return infos;
    });
}
//# sourceMappingURL=upload.js.map