var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
import { Buffer } from "buffer";
export const SPEC_URL = "https://git-lfs.github.com/spec/v1";
export const LFS_POINTER_PREAMBLE = `version ${SPEC_URL}\n`;
/** Returns true if given blob represents an LFS pointer. */
export function pointsToLFS(content) {
    return (content[0] === 118 && // 'v'
        // TODO: This is inefficient, it should only search the first line or first few bytes.
        content.indexOf(LFS_POINTER_PREAMBLE) === 0);
}
/**
 * Returns properly encoded HTTP Basic auth header,
 * given basic auth credentials.
 */
export function getAuthHeader(auth) {
    return {
        Authorization: `Basic ${Buffer.from(`${auth.username}:${auth.password}`).toString("base64")}`,
    };
}
/**
 * Returns true if given path is available for writing,
 * regardless of whether or not it is occupied.
 */
export function isWriteable({ promises: fs }, filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // TODO: there's no API for this in PromiseFsClient world
            // await fs.access(filepath, fsConstants.W_OK);
            return true;
        }
        catch (e) {
            if (e.code === "ENOENT") {
                return true;
            }
            return false;
        }
    });
}
/**
 * Returns true if given path is available for writing
 * and not occupied.
 */
export function isVacantAndWriteable(filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // TODO: there's no API for this in PromiseFsClient world
            return true;
            // await fs.access(filepath, fsConstants.W_OK);
        }
        catch (e) {
            if (e.code === "ENOENT") {
                return true;
            }
        }
        return false;
    });
}
export function bodyToBuffer(body) {
    var _a, body_1, body_1_1;
    var _b, e_1, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const buffers = [];
        let offset = 0;
        let size = 0;
        try {
            for (_a = true, body_1 = __asyncValues(body); body_1_1 = yield body_1.next(), _b = body_1_1.done, !_b;) {
                _d = body_1_1.value;
                _a = false;
                try {
                    const chunk = _d;
                    buffers.push(chunk);
                    size += chunk.byteLength;
                }
                finally {
                    _a = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_a && !_b && (_c = body_1.return)) yield _c.call(body_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        const result = new Uint8Array(size);
        for (const buffer of buffers) {
            result.set(buffer, offset);
            offset += buffer.byteLength;
        }
        return Buffer.from(result.buffer);
    });
}
// Borrowed from Isomorphic Git core, it is not importable.
export function toHex(buffer) {
    let hex = "";
    for (const byte of new Uint8Array(buffer)) {
        if (byte < 16)
            hex += "0";
        hex += byte.toString(16);
    }
    return hex;
}
//# sourceMappingURL=util.js.map